"
 COPYRIGHT (c) 1992-93 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

NonPositionableExternalStream subclass:#Socket
       instanceVariableNames:'domain socketType protocol portNr serviceName'
       classVariableNames:''
       poolDictionaries:''
       category:'Streams-External'
!

Socket comment:'

COPYRIGHT (c) 1992-93 by Claus Gittinger
              All Rights Reserved

I provide access to sockets for interprocess communication.

My protocol is in analogy to the unix-socket interface:

        create:
            aSocket := Socket domain:#INET type:#STREAM protocol:''tcp''

        bind:
            retVal := aSocket bind:aName
        
%W% %E%
'!

%{
#include <stdio.h>
#ifdef LINUX
/* kludge to avoid some redefines ... */
# define _ARPA_NAMESER_H
# define _NETINET_TCP_H
#endif
#ifndef transputer
# include <fcntl.h>
# include <sys/types.h>
# include <sys/socket.h>
# include <netdb.h>
# include <netinet/in.h>
# if ! (defined(SYSV3) && defined(mc88k))
#  include <netinet/tcp.h>
# endif
#endif
%}

!Socket class methodsFor:'instance creation'!

connectTo:service on:host
    "create new client socket, bind and connect; return the socket"

    |port newSocket|

    port := self portOfService:service.
    newSocket := (self basicNew) for:host port:port.
    ^ newSocket

    "Socket connectTo:9995 on:'clam'"
    "Socket connectTo:'finger' on:'clam'"
!

provide:service
    "create a new server socket providing a service"

    |port newSocket|

    port := self portOfService:service.
    newSocket := (self basicNew) on:port.
    ^ newSocket

    "Socket provide:9995"
!

acceptFrom:aSocket
    "create a new socket from accepting on another (server)
     socket"

    |newSocket|

    newSocket := self basicNew.
    ^ newSocket acceptOn:aSocket

    "Socket acceptFrom:(Socket provide:9995)"
!

domain:domainSymbol type:type protocol:aStringOrNumberOrNil
    ^ self basicNew domain:domainSymbol type:type protocol:aStringOrNumberOrNil

    "Socket domain:#INET type:#DGRAM protocol:nil"
! !

!Socket class methodsFor:'queries'!

portOfService:aNameOrNumber
    "returns the port-number for a given service
     or nil if no such service exists;
     - used to convert service names to portNumbers"
%{
    struct servent *servent = NULL;

    if (_isSmallInteger(aNameOrNumber)) {
        servent = getservbyport(htons(_intVal(aNameOrNumber)), "tcp") ;
        if (servent != NULL) {
            RETURN ( aNameOrNumber );
        }
        RETURN ( aNameOrNumber );
    }
    if (_isString(aNameOrNumber)) {
        servent = getservbyname((char *) _stringVal(aNameOrNumber), "tcp");
        if (servent != NULL) {
            RETURN ( _MKSMALLINT(servent->s_port) );
        }
        RETURN ( nil );
    }
    RETURN ( nil );
%}
    "Socket portOfService:'finger'"
!

protocolOfService:aNameOrNumber
    "returns the protocol (as string) for a given service
     or nil if no such service exists"
%{
    struct servent *servent = NULL;

    if (_isSmallInteger(aNameOrNumber)) {
        servent = getservbyport(htons(_intVal(aNameOrNumber)), "tcp") ;
        if (servent == NULL) {
            servent = getservbyport(htons(_intVal(aNameOrNumber)), "udp") ;
            if (servent == NULL) {
                RETURN ( nil );
            }
        }
    } else {
        if (_isString(aNameOrNumber)) {
            servent = getservbyname((char *) _stringVal(aNameOrNumber), "tcp");
            if (servent == NULL) {
                servent = getservbyname((char *) _stringVal(aNameOrNumber), "udp");
                if (servent == NULL) {
                    RETURN ( nil );
                }
            }
        }
    }
    if (servent) {
        RETURN ( _MKSTRING(servent->s_proto COMMA_CON) );
    }
    RETURN ( nil );
%}
    "Socket protocolOfService:'finger'"
    "Socket protocolOfService:79"
! !

!Socket methodsFor:'socket setup'!

domain:domainSymbol type:type protocol:aStringOrNumberOrNil
    "set up socket with domain, type and protocol"

    "{ Symbol: UNIX }"
    "{ Symbol: INET }"
    "{ Symbol: NS }"
    "{ Symbol: APPLETALK }"
    "{ Symbol: X25 }"

    "{ Symbol: STREAM }"
    "{ Symbol: DGRAM }"
    "{ Symbol: RAW }"
    "{ Symbol: SEQPACKET }"

    domain := domainSymbol.
    socketType := type.
    protocol := aStringOrNumberOrNil.
%{
    FILE *fp;
    int dom, typ, proto, sock;
    char *protoName;
    struct protoent *protoEntry;
    extern struct protoent *getprotobyname();

    /*
     * get address-family
     */
#ifdef AF_UNIX
    if (domainSymbol == _UNIX)
        dom = AF_UNIX;
    else
#endif
#ifdef AF_INET
    if (domainSymbol == _INET)
        dom = AF_INET;
    else
#endif
#ifdef AF_NS
    if (domainSymbol == _NS)
        dom = AF_NS;
    else
#endif
#ifdef AF_APPLETALK
    if (domainSymbol == _APPLETALK)
        dom = AF_APPLETALK;
    else
#endif
#ifdef AF_X25
    if (domainSymbol == _X25)
        dom = AF_X25;
    else
#endif
    {
        printf("bad domain\n");
        RETURN ( nil );
    }

#ifdef SOCK_STREAM
    if (type == _STREAM)
        typ = SOCK_STREAM;
    else
#endif
#ifdef SOCK_DGRAM
    if (type == _DGRAM)
        typ = SOCK_DGRAM;
    else 
#endif
#ifdef SOCK_RAW
    if (type == _RAW)
        typ = SOCK_RAW;
    else
#endif
#ifdef SOCK_SEQPACKET
    if (type == _SEQPACKET)
        typ = SOCK_SEQPACKET;
    else
#endif
    {
        printf("bad type\n");
        RETURN ( nil );
    }

    if (_isString(aStringOrNumberOrNil)) {
        protoEntry = getprotobyname((char *) _stringVal(aStringOrNumberOrNil));
        if (protoEntry == NULL) {
            RETURN ( nil );
        }
        proto = protoEntry->p_proto;
        endprotoent();
    } else if (_isSmallInteger(aStringOrNumberOrNil)) {
        proto = _intVal(aStringOrNumberOrNil);
    } else {
        proto = 0;
    }

    sock = socket(dom, typ, proto);
    if (sock >= 0) {
        /* make it a FILE * */
        fp = fdopen(sock, "r+");
        if (fp) {
            _INST(filePointer) = MKOBJ(fp);
            RETURN ( self );
        }
    }
    RETURN ( nil );
%}
    "Socket domain:#INET type:#STREAM protocol:nil"
!

onPort:portNr 
    "setup for a server socket"

    ^ self socketOnPort:portNr withBackLog:1
!

onPort:portNr withBackLog:aNumber
    "setup for a server socket"
%{
    struct sockaddr_in sa ;
    int sock;
    FILE *fp;
    int backLog;

    if (! _isSmallInteger(portNr)) {
        RETURN ( nil );
    }

    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        RETURN ( nil );
    }

    bzero((char *) &sa, sizeof(sa)) ;
    sa.sin_family = AF_INET ;
    sa.sin_addr.s_addr = htonl(INADDR_ANY) ;
    sa.sin_port = htons(_intVal(portNr)) ;

    if (bind(sock, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
        printf("bind failed\n");
        close(sock);
        RETURN ( nil );
    }

    if (_isSmallInteger(aNumber)) {
        backLog = _intVal(aNumber);
    } else
        backLog = 1;

    if (listen(sock, backLog) < 0) {
        printf("listen failed\n");
        close(sock);
        RETURN ( nil );
    }

    /* make it a FILE * */
    fp = fdopen(sock, "r+");
    if (! fp) {
        close(sock);
        RETURN ( nil );
    }
    _INST(filePointer) = MKOBJ(fp);
    RETURN ( self );
%}
!

for:hostName udpPort:portNr
    "setup for a UDP socket - if hostname is nil, a server port is opened;
     otherwise a client port to the server on host."
%{
    struct sockaddr_in sa ;
    struct hostent *hp ;
    int a, sock ;
    long addr;
    FILE *fp;

    if (! _isSmallInteger(portNr)) {
        RETURN ( nil );
    }

    if (hostName != nil) {
        bzero(&sa, sizeof(sa)) ;
        if ((addr = inet_addr((char *) _stringVal(hostName))) != -1) {
            /* is Internet addr in octet notation */
            bcopy(&addr, (char *) &sa.sin_addr, sizeof(addr)) ; /* set address */
            sa.sin_family = AF_INET ;
        } else {
            /* do we know the host's address? */
            if ((hp = gethostbyname((char *) _stringVal(hostName))) == NULL) {
                printf("unknown host\n");
                RETURN ( nil );
            }
            bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length) ;
            sa.sin_family = hp->h_addrtype ;
        }
    } else
        sa.sin_family = AF_INET;

    sock = socket(sa.sin_family, SOCK_DGRAM, 0);
    if (sock < 0) {
        printf("socket failed\n");
        RETURN ( nil );
    }

    if (hostName == nil) {
        sa.sin_addr.s_addr = htonl(INADDR_ANY);
        if (bind(sock, &sa, sizeof(sa)) < 0) {
            close(sock) ;
            RETURN ( nil );
        }
    } else {
        sa.sin_port = htons((u_short) _intVal(portNr)) ;
        if (connect(sock, &sa, sizeof(sa)) < 0) {
            close(sock) ;
            RETURN ( nil );
        }
    }

    /* make it a FILE * */
    fp = fdopen(sock, "r+");
    if (! fp) {
        close(sock);
        RETURN ( nil );
    }
    _INST(filePointer) = MKOBJ(fp);
    RETURN ( self );
%}
!

for:hostName port:portNr
    "setup for a tcp socket. If hostname is nil, a server port is opened
     otherwise a client port to the server on host."
%{
    struct sockaddr_in sa ;
    struct hostent *hp ;
    int a, sock ;
    long addr;
    FILE *fp;

    if (! _isSmallInteger(portNr)) {
        RETURN ( nil );
    }

    if (hostName != nil) {
        bzero(&sa, sizeof(sa)) ;
        if ((addr = inet_addr((char *) _stringVal(hostName))) != -1) {
            /* is Internet addr in octet notation */
            bcopy(&addr, (char *) &sa.sin_addr, sizeof(addr)) ; /* set address */
            sa.sin_family = AF_INET;
        } else {
            /* do we know the host's address? */
            if ((hp = gethostbyname((char *) _stringVal(hostName))) == NULL) {
                printf("unknown host\n");
                RETURN ( nil );
            }
            bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length) ;
            sa.sin_family = hp->h_addrtype;
        }
    } else
        sa.sin_family = AF_INET;

    sock = socket(sa.sin_family, SOCK_STREAM, 0); 
    if (sock < 0) {
        printf("socket failed\n");
        RETURN ( nil );
    }

    /* connect */
    sa.sin_port = htons((u_short) _intVal(portNr)) ;
    if (hostName != nil) {
        if (connect(sock, &sa, sizeof(sa)) < 0) {
            close(sock);
            RETURN ( nil );
        }
    } else {
        sa.sin_addr.s_addr = htonl(INADDR_ANY);
        if (bind(sock, &sa, sizeof(sa)) < 0) { 
            close(sock) ;
            RETURN ( nil );
        }
    }

    /* make it a FILE * */
    fp = fdopen(sock, "r+");
    if (! fp) {
        close(sock);
        RETURN ( nil );
    }
    _INST(filePointer) = MKOBJ(fp);
    RETURN ( self );
%}
! !

!Socket methodsFor:'low level'!

bindTo:portNumber address:address
%{
    OBJ fp = _INST(filePointer);
    int fd;
    int family;
    struct sockaddr_in insock;
    int ok = 1;

    if (_INST(domain) == _INET) {
        insock.sin_family = AF_INET;
        if (_isSmallInteger(portNumber)) {
            insock.sin_port = htons(_intVal(portNumber));
            if (address == nil) {
                insock.sin_addr.s_addr = htonl(INADDR_ANY);
            } else
                ok = 0;
        } else
            ok = 0;
    } else
        ok = 0;

    if (ok) {
        if (fp != nil) {
            fd = fileno(MKFD(fp));
            if (bind(fd, (struct sockaddr *) &insock, sizeof (insock))) {
                RETURN ( false) ;
            }
            RETURN ( true );
        }
    }
%}
.
    ^ false

    "(Socket domain:#INET type:#STREAM protocol:nil)
         bindTo:port
         address:nil"
!

listenWithBacklog:aNumber
%{
    OBJ fp = _INST(filePointer);
    int fd;

    if (fp != nil) {
        fd = fileno((FILE *)(_intVal(fp)));
        if (_isSmallInteger(aNumber)) {
            listen(fd, _intVal(aNumber));
            RETURN (true );
        }
    } 
%}
.
    ^ false
!

acceptOn:aSocket
    "accept a connection on a server port (created with:'Socket>>onPort:')
     usage is: (Socket basicNew acceptOn:(Socket onPort:9999))"

    |fileDes|

    filePointer notNil ifTrue:[
        self error:'already connected'
    ].
    fileDes := aSocket fileDescriptor.
%{
    FILE *fp;
    int flags;
    int fd, newFd;
    struct sockaddr_in sa ;
    int alen;
    struct hostent *he ;
    long norder ;
    char dotted[20] ;

    alen = sizeof(sa) ;
    if (_isSmallInteger(fileDes)) {
        fd = _intVal(fileDes);
        flags = ioctl(fd, F_GETFL, 0);
        ioctl(fd, F_SETFL, flags | O_NDELAY);
        newFd = accept(fd, (struct sockaddr *) &sa, &alen);
        if (newFd >= 0) {
            he = gethostbyaddr(&sa.sin_addr.s_addr, alen, AF_INET) ;
            if (! he) {
                norder = htonl(sa.sin_addr.s_addr) ;
                sprintf(dotted, "%d.%d.%d.%d",
#if defined(SYSV3) || defined(LINUX) || defined(IRIS)
                        (sa.sin_addr.s_addr >> 24) & 0xFF,
                        (sa.sin_addr.s_addr >> 16) & 0xFF,
                        (sa.sin_addr.s_addr >> 8) & 0xFF,
                        sa.sin_addr.s_addr & 0xFF);
#else
                        sa.sin_addr.s_net, sa.sin_addr.s_host,
                        sa.sin_addr.s_lh, sa.sin_addr.s_impno);
#endif
            }
            fprintf(stderr, "accepted connection from host %s\n",
                    (he ? he->h_name : dotted)) ;

            /* make it a FILE * */
            fp = fdopen(newFd, "r+");
            if (fp) {
                _INST(filePointer) = MKOBJ(fp);
            }
            RETURN ( self );
        }
    }
    RETURN ( nil );
%}
! !

!Socket methodsFor:'redefined basic'!

size
    ^ self shouldNotImplement
!

position:newpos
    ^ self shouldNotImplement
!

position
    ^ self shouldNotImplement
! !
